# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s):
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: $
"""Interface to the quickGO interface

.. topic:: What is quickGO

    :URL: http://www.ebi.ac.uk/QuickGO/
    :Service: http://www.ebi.ac.uk/QuickGO/WebServices.html

    .. highlights::

        "QuickGO is a fast web-based browser for Gene Ontology terms and
        annotations, which is provided by the UniProt-GOA project at the EBI. "

        -- from QuickGO home page, Dec 2012

"""
from __future__ import print_function

from bioservices.services import RESTService

__all__ = ["QuickGO"]

class QuickGO(RESTService):
    """Interface to the `QuickGO <http://www.ebi.ac.uk/QuickGO/WebServices.html>`_ service

    .. doctest::

        >>> from bioservices import QuickGO
        >>> s = QuickGO()
        >>> res = s.Term("GO:0003824")

    """
    _goid_example = "GO:0003824"

    def __init__(self, verbose=True):
        """.. rubric:: Constructor

        :param bool verbose: print informative messages.

        """
        super(QuickGO, self).__init__(url="http://www.ebi.ac.uk/QuickGO",
            name="quickGO", verbose=verbose)

    def Term(self, GOid, format="oboxml"):
        """Obtain Term information


        :param str format: the output format (mini, obo, oboxml).

        The format can be:

        * mini:   Mini HTML, suitable for dynamically embedding in popup boxes.
        * obo:    OBO format snippet.
        * oboxml: OBO XML format snippet.

        ::

            from bioservices import QuickGO
            s = QuickGO()
            s.Term("GO:0003824")


        """
        _valid_formats = ["mini", "obo", "oboxml"]
        if GOid.startswith("GO:")==False:
            raise ValueError("GO id must start with 'GO:'")

        if format not in _valid_formats:
            raise ValueError("format provided is incorrect. Choose in %s" % _valid_formats)

        url = self.url + "/GTerm?id=" + GOid
        #GO:0003824
        params = {'format':format}
        postData = self.urlencode(params)
        url += "&" + postData

        if format in ["oboxml"]:
            res = self.request(url, format="xml")
        else:
            res = self.request(url, format=format)

        return res



    def Annotation(self, goid=None, protein=None, format="gaf", limit=10000,
        gz=False, col=None, db=None, aspect=None, relType=None, termUse=None,
        evidence=None, source=None, ref=None,  tax=None, qualifier=None, q=None, _with=None):
        """Calling the Annotation service

        Mutual exclusive parameters are goid, protein

        :param int limit: download limit (number of lines) (default 10,000 rows,
            which may not be sufficient for the data set that you are
            downloading. To bypass this default, and return the entire data set,
            specify a limit of -1).
        :param bool gz: gzips the downloaded file.
        :param str goid: GO identifiers either directly or indirectly
            (descendant GO identifiers) applied in annotations.
        :param char aspect: use this to limit the annotations returned to a
            specific ontology or ontologies (Molecular Function, Biological
            Process or Cellular Component). The valid character can be F,P,C.
        :param relType: not Implemented. By default, QuickGO will display annotations to GO terms
            that are related to that specified in the goid parameter by is_a,
            part_of and occurs_in relations; this parameter allows you to
            override that behaviour. See `details <http://www.ebi.ac.uk/QuickGO/reference.html#slim_custom>`_
        :param termUse:  if you set this parameter to slim, then QuickGO will
            use the supplied set of GO identifiers as a slim and will map the
            annotations up to these terms. See here for more details:
            http://www.ebi.ac.uk/QuickGO/GMultiTerm
        :param str db: protein database (identifier type). Can be UniProtKB, UniGene, Ensembl.
        :param evidence: annotation evidence code category (Ev). Example of
            valid evidence are: be IDA, IC, ISS, IEA, IPI, ND, IMP, ISO, IGI
            should be either a string with comma separated values (e.g.,
            IEA,IDA) or a list of strings (e.g. ["IEA","IDA"]).
        :param source: annotation provider. Examples are 'InterPro', 'UniPathway',
            'MGI', 'FlyBase', 'GOC', 'Source', 'UniProtKB', 'RGD', 'ENSEMBL',
            'ZFIN', 'IntAct'.
        :param ref: PubMed or GO reference supporting annotation. Can refer to a
            specific reference identifier or category (for category level, use
            `*`  after ref type). Can be 'PUBMED:`*`', 'GO_REF:0000002'.
        :param with: additional supporting information supplied in IEA, ISS, IPI, IC
            evidenced annotations; see GO documentation. Can refer to a specific
            identifier or category (for category level, use * after with type).
            Examples are: EC:2.5.1.30, IPR000092, HAMAP:*
        :param str qualifier: tags that modify the interpretation of an annotation.
             Examples are NOT, colocalizes_with, contributes_to.

        .. note::
            * Any number of fields can be specified; they will be AND'ed together.
            * Any number of values can be specified for each field; they will be OR'ed together.
            * Values should be URI encoded.
            * The file will be truncated if more than the specified number of annotations are found. The file is roughly 170 bytes/annotation (not gzipped).
            * The file will be gzipped if the gz parameter is supplied.


        ::

            >>> print s.Annotation(protein='P12345', format='tsv', col="ref,evidence",
            ... ref='PMID:*')
            >>> print s.Annotation(protein='P12345,Q4VCS5', format='tsv', 
            ...     col="ref,evidence",ref='PMID:,Reactome:')



        """
        _valid_formats = ["gaf", "gene2go", "proteinList", "fasta", "tsv"]
        _valid_col = ['proteinDB', 'proteinID', 'proteinSymbol', 'qualifier',
            'goID', 'goName', 'aspect', 'evidence', 'ref', 'with', 'proteinTaxon',
            'date', 'from', 'splice', 'proteinName', 'proteinSynonym', 'proteinType',
            'proteinTaxonName', 'originalTermID', 'originalGOName']
        _valid_db = ['UniProtKB', 'UniGene', 'Ensembl']
        _valid_aspect = ['P', 'F', 'C']

        if isinstance(limit, int)==False:
            raise TypeError("limit parameter must be an integer greater than zero")

        # fill params with parameters that have default values.
        params = {'format':format, 'limit':limit}

        # beginning of the URL
        url = self.url + "/GAnnotation?"

        # what is the ID being provided. We can have only one of:
        # protein, goid
        if protein != None:
            url += "protein=" + protein
        elif goid != None:
            url += "goid=" + goid
        elif tax != None:
            url += "tax=" + tax

        #need to check that there are mutualy exclusive
        if goid==None and protein==None and tax==None:
             raise ValueError("you must provide at least one of the following parameter: goid, protein")

        # aspect parameter
        if aspect != None:
            if aspect not in _valid_aspect:
                 raise ValueError("Invalid aspect (%s) must be in %s" % (aspect, _valid_aspect))
            params['aspect'] = aspect

        # aspect parameter
        if termUse != None:
            self.checkParam(termUse, ["slim"])
            params['termUse'] = termUse

        if relType:
             raise NotImplementedError

        if q:
             raise NotImplementedError

        if evidence:
            if isinstance(evidence,list):
                evidence = ",".join([x.strip() for x in evidence])
            elif isinstance(evidence,str):
                pass
            else:
                raise ValueError("""
Invalid parameter: evidence parameters must be a list of strings ['IDA','IEA']
or a string (e.g., 'IDA', 'IDA,IEA') """)
            params['evidence'] = evidence

        if source:
            if isinstance(source, list):
                source = ",".join([x.strip() for x in source])
            elif isinstance(source, str):
                pass
            else:
                raise ValueError("""
Invalid parameter: source parameters must be a list of strings ['UniProtKB']
or a string (e.g., 'UniProtKB') """)
            params['source'] = source

        if ref:
            if isinstance(ref, list):
                ref = ",".join([x.strip() for x in ref])
            elif isinstance(ref, str):
                pass
            else:
                raise ValueError("""
Invalid parameter: source parameters must be a list of strings ['PUBMED']
or a string (e.g., 'PUBMED:*') """)
            params['ref'] = ref


        if qualifier:
            #NOT, colocalizes_with, contributes_to
            if isinstance(qualifier, list):
                qualifier = ",".join([x.strip() for x in qualifier])
            elif isinstance(qualifier, str):
                pass
            params['qualifier'] = qualifier


        # col parameter
        if format=="tsv":
            if col == None:
                col = 'proteinDB,proteinID,proteinSymbol,qualifier,'
                col += 'goID,goName,aspect,evidence,ref,with,proteinTaxon,date,from,splice'
            for c in col.split(','):
                if c not in _valid_col:
                    raise ValueError("col parameter error: found %s, which is not a valid value." % c)
            params["col"] = col
        if format!="tsv":
            # col is provided but format is not appropriate
            if col!= None:
                raise ValueError("You provided the 'col' parameter but the format is not correct. You should use the 'format=tsv")

        postData = "&" + self.urlencode(params)

        # gz parameter. do not expect values so need to be added afterwards.
        if gz==True:
            url+='&gz'

        url += postData
        res = self.request(url, format="txt")
        return res







from bioservices.services import *


class test_Service(Service):
    def __init__(self):
        super(test_Service, self).__init__("test", "http://www.uniprot.org", verbose=False)
        self.url
        self.easyXMLConversion

    def test_easyXML(self):
        self.easyXML("<xml><id></id></xml>")

    def test_urlencode(self):
        res = self.urlencode({'a':1, 'b':2})
        assert res in ["a=1&b=2", "b=2&a=1"]

class test_WSDLService(WSDLService):
    def __init__(self):
        super(test_WSDLService, self).__init__("test",
            "http://www.ebi.ac.uk/biomodels-main/services/BioModelsWebServices?wsdl", 
            verbose=False)

    def test_methods(self):
        self.methods
    def test_dump(self):
        self.dumpOut
        self.dumpIn
        self.dumpOut = 1
        self.dumpIn = 1



class test_RESTService(RESTService):
    def __init__(self):
        super(test_RESTService, self).__init__("test",
            "http://www.uniprot.org", 
            verbose=False)

    def test_request(self):
        try:
            self.request("dummy")
            assert False
        except:
            assert True



from bioservices.wikipathway import  Wikipathway
import unittest


class TestRepGen(unittest.TestCase, Wikipathway):
    @classmethod
    def setUpClass(self):
        self.s = Wikipathway()




class test_wiki(TestRepGen):
    @classmethod
    def setUpClass(self):
        super(test_wiki, self).setUpClass()


    def test_organism(self):
        assert len(self.s.organisms)
        self.s.organism = 'Homo sapiens'
        assert self.s.organism == 'Homo sapiens'

        try:
            self.s.organism = 'Homo sapi'
            assert False
        except ValueError:
            assert True

    @unittest.skip("test")
    def test_showPathwayInBrowser(self):
        self.s.showPathwayInBrowser("WP2320")


    #@unittest.skip("test")
    def test_listPathways(self): 
        l = self.s.listPathways()
        #len(l) > 40
        l = self.s.listPathways("Homo sapiens")
        #len(l) > 40

    def test_getPathway(self):
        self.s.getPathway("WP2320")
    def test_getPathwayInfo(self):
        self.s.getPathwayInfo("WP2320")

    def test_getPathwayAs(self):
        res = self.s.getPathwayAs("WP4", filetype="txt")
        


    def test_findPathwaysByText(self):
        res = self.s.findPathwaysByText(query="p53")
        res = self.s.findPathwaysByText(query="p53",species='Homo sapiens')
        len([x.score for x in res]) == len(res)
        assert len(self.s.findPathwaysByText(query="p53 OR mapk",species='Homo sapiens'))>0

    def test_getOntologyTersmByPathway(self):
        res = self.s.getOntologyTermsByPathway("WP4")
        res[0].ontology

    def test_getOntologyTermsByOntology(self):
        self.s.getOntologyTermsByOntology("Disease")

    def test_getPathwaysByOntologyTerm(self):
        self.s.getPathwaysByOntologyTerm('DOID:344')


    def test_getPathwaysByParentOntologyTerm(self):
        self.s.getPathwaysByParentOntologyTerm("DOID:344")


    def test_getCurationTags(self):
        self.s.getCurationTags("WP4")

    def test_getcurationTagByNames(self):
        self.s.getCurationTagsByName("Curation:Tutorial")

    def test_findInteractions(self):
        self.s.findInteractions("P53")
        self.s.findInteractions("P53", interactionOnly=False)
        self.s.findInteractions("P53", raw=True)

    def test_getRecentChanges(self):
        self.s.getRecentChanges(20120101000000)

    def test_findPathwayByXref(self):
        self.s.findPathwaysByXref('P45985')


    def test_findPathwaysByLitterature(self):
        self.s.findPathwaysByLiterature(18651794)

    def test_savePathwayAs(self):
        self.s.savePathwayAs("WP4", "test.pdf", display=False)
        import os
        try:os.remove("test.pdf")
        except:pass
    def test_getPathwaysByParentOntologyTerm(self):
        self.s.getPathwaysByParentOntologyTerm("DOID:344")


    def test_createPathway(self):
        try:
            self.s.createPathway("","")
            assert False
        except NotImplementedError:
            assert True


    def test_updatePathwa(self):
        try:
            self.s.updatePathway("","","")
            assert False
        except NotImplementedError:
            assert True

    def test_saveCurationTag(self):
        try:
            self.s.saveCurationTag("","","")
            assert False
        except NotImplementedError:
            assert True

    def test_login(self):
        try:
            self.s.login("dummy", "dummy")
            assert False
        except NotImplementedError:
            assert True
    def test_remoceCurationTag(self):
        try:
            self.s.removeCurationTag("dummy", "dummy")
            assert False
        except NotImplementedError:
            assert True

    def test_getPathwayHistory(self):
        res = self.s.getPathwayHistory("WP455", "20100101000000")

    def test_coloredPathway(self):
        res = self.s.getColoredPathway("WP4",revision=0)






from bioservices import Kegg, KeggParser
from nose import with_setup



class test_Kegg(Kegg):

    def __init__(self):
        super(test_Kegg, self).__init__(verbose=True)
        #self.test_database_IDs()


    def test_database_IDs(self):
        self.organismIds
        assert self.enzymeIds[0].startswith("ec")
        assert self.compoundIds[0].startswith("cpd")
        assert self.glycanIds[0].startswith("gl")
        assert self.reactionIds[0].startswith("rn")
        assert self.drugIds[0].startswith("dr")
        assert self.koIds[0].startswith("ko")

    def test_organism(self):
        self.organism = "hsa"
        try:
            self.organism = "dummy"
            assert False
        except:
            assert True

    def test_isOrganism(self):
        assert self.isOrganism('T01440') == True
        assert self.isOrganism('hsa') == True
        assert self.isOrganism('dummy') == False

    def test_pathwayIDs(self):
        self.organism = "hsa"
        self.pathwayIds

    def test_info(self):
        self.info()
        self.info("brite")
        self.info("hsa")
        try:
            self.info("dummy")
            assert False
        except:
            assert True


    def test_list(self):
        #self.list("pathway")             # returns the list of reference pathways
        self.list("pathway", "hsa")      # returns the list of human pathways
        self.list("organism")            # returns the list of KEGG organisms with taxonomic classification
        #self.list("hsa")                 # returns the entire list of human genes
        #self.list("T01001")              # same as above
        self.list("hsa:10458+ece:Z5100") # returns the list of a human gene and an E.coli O157 gene
        self.list("cpd:C01290+gl:G00092")# returns the list of a compound entry and a glycan entry
        self.list("C01290+G00092")       # same as above 

        # invalid queries:
        try:
            self.list("drug", "hsa")
            assert False
        except:
            assert True

        try:
            self.list("dumy")
            assert False
        except:
            assert True

    def test_find(self):
        self.find("compound", "300-310", "mol_weight")
        self.find("genes", "shiga+toxin")             # for keywords "shiga" and "toxin"
        self.find("genes", "shiga toxin")            # for keywords "shigatoxin"
        self.find("compound", "C7H10O5", "formula")   # for chemicalformula "C7H10O5"
        self.find("compound", "O5C7","formula")       # for chemicalformula containing "O5" and "C7"
        self.find("compound", "174.05","exact_mass")  # for 174.045 =<exact mass < 174.055
        self.find("compound", "300-310","mol_weight") # for 300 =<molecular weight =< 310 

    def test_get(self):
        self.get("cpd:C01290+gl:G00092")
        self.get("C01290+G00092")
        self.get("hsa:10458+ece:Z5100")
        self.get("hsa:10458+ece:Z5100", "aaseq") 
        res = self.get("hsa05130", "image")      
        try:
            self.get("hsa05130", "imagffe")      
            assert False
        except:
            assert True


    def test_conv(self):

        try:
            self.conv("unipro", "hsa")
            assert False
        except:
            assert True

        try:
            self.conv("uniprot", "hs")
            assert False
        except:
            assert True

        try:
            self.conv("hs", "unipro")
            assert False
        except:
            assert True

        try:
            self.conv("hsa", "unipr")
            assert False
        except:
            assert True

        # asc contains 1500. Try to get even samller to spped up tests.
        #self.conv("asc", "uniprot")


        self.conv("hsa","up:Q9BV86+")

    def _test_show_pathway(self):
        self.show_entry("path:hsa05416")
        self.show_pathway("path:hsa05416", scale=50)

    def test_check_dentries(self):
        assert True == self.check_dbentries("hsa:10458+ece:Z5100")
        try:
            self.check_dbentries("hsa:10458+ece:Z510", checkAll=False)
            assert False
        except:
            assert True

    def test_link(self):
        self.link("pathway", "hsa:10458+ece:Z5100")


    def test_org_conv(self):
        assert 'hsa' == self.Tnumber2code("T01001")
        assert 'T01001' == self.code2Tnumber("hsa")

    def test_parse_kgml_pathway(self):
        res = self.parse_kgml_pathway("hsa04660")


def test_KeggParser():
    s = KeggParser()
    res = s.get("rp:RP00001")
    d = s.parseRpair(res)



from bioservices import *


class test_nciblast(NCBIblast):
    def __init__(self):
        super(test_nciblast, self).__init__(verbose=False)

    def test_param(self):
        self.getParameters()
        assert(len(self.parameters)>0)
        assert(len(self.parameters)>0)

    def test_paramdetails(self):
        names = self.parametersDetails("matrix") 
        try:
            names = self.parametersDetails("matrixddddd") 
            assert False
        except:
            assert True

    def test_run(self):

        try:
            self.jobid = self.run(program="blastp", sequence=self._sequence_example,
                stype="protein", database="uniprotkb")
            assert False # missing argument
        except:
            assert True
        self.jobid = self.run(program="blastp", sequence=self._sequence_example,
            stype="protein", database="uniprotkb", email="name@test.org",
            matrix="BLOSUM45")
        res = self.getResult(self.jobid, "out")

        res = self.getResultTypes(self.jobid)

    def test_attributse(self):
        self.databases



# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s): 
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: kegg.py 156 2013-02-17 22:45:39Z cokelaer $
"""Interface to the ArrayExpress web Service.

.. topic:: What is ChemSpider ?


    :URL:  http://www.chemspider.com/
    :REST: http://www.chemspider.com/AboutServices.aspx?

    .. highlights::

        ChemSpider is a free chemical structure database providing fast access to 
        over 28 million structures, properties and associated information. By 
        integrating and linking compounds from more than 400 data sources, ChemSpider 
        enables researchers to discover the most comprehensive view of freely 
        available chemical data from a single online search. It is owned by the Royal Society of Chemistry.

        -- ChemSpider home page, March 2013
"""
from bioservices import RESTService


class ChemSpider(RESTService):
    """ChemSpider Web Service Interface

    :status: in progress you can already search for Id and compound or 
        retrieve the chemical image of an Id


    ::

        >>> from bioservices import *
        >>> s = ChemSpider()
        >>> s.find("Pyridine")
        1020
        >>> results = s.GetExtendedCompoundInfo(1020)
        >>> results['averagemass']
        79.0999


    """
    def __init__(self, verbose=False):
        self._token = "7a621478-4e35-4da0-895c-7950af45f2fd"
        url = 'http://www.chemspider.com/'
        super(ChemSpider, self).__init__("ChemSpider", url=url, verbose=verbose)
        self._databases = None

    def _set_token(self, token):
        self._token = token
    def _get_token(self):
        return self._token
    token = property(None, _set_token)

    def find(self, query):
        """return the first 100 compounds that match the query"""
        import urllib2
        this = "Search.asmx/SimpleSearch?query=%s&token=%s" % (urllib2.quote(query), self._token)
        res = self.request(this)
        Ids = [int(x.text) for x in res.findAll("int")]
        return Ids

    def GetExtendedCompoundInfo(self, Id):
        url = self.url + "/MassSpecAPI.asmx/GetExtendedCompoundInfo?CSID=%s&token=%s" % (Id, self._token) 
        res = self.request(url)

        results = {}
        data = [(x.tag, x.text) for x in res.getchildren()]
        for datum in data:
            tag, value = datum
            ID = tag.split("{http://www.chemspider.com/}")[1]
            try:
                value = float(value)
            except:
                pass
            results[ID.lower()] = value
        return results

    def ImagesHandler(self, Id):
        url = self.url + "/ImagesHandler.ashx?id=%s"  % Id
        res = self.request(url)
        return res

    def image(self, Id):
        """ Return string containing PNG binary image data of 2D structure image 

        ::

            >>> from bioservices import *
            >>> s = ChemSpider()
            >>> ret = s.image(1020)
            >>> with open("test.png", "w") as f:
            ...     f.write(ret)
            >>> s.onWeb("test.png")
        """
        url = self.url + "/Search.asmx/GetCompoundThumbnail?id=%s&token=%s" % (Id, self._token)
        res = self.request(url)
        import base64
        image = base64.b64decode(res.root.text)
        return image

    def mol(self, Id):
        """ Return record in MOL format """
        url = self.url + '/MassSpecAPI.asmx/GetRecordMol?csid=%s&calc3d=false&token=%s' % (Id, self._token)
        ret = self.request(url)
        return ret.root.text

    def mol3d(self, Id):
        """ Return record in MOL format with 3D coordinates calculated """
        url = '/MassSpecAPI.asmx/GetRecordMol?csid=%s&calc3d=true&token=%s' % (Id, self._token)
        ret = self.request(url)
        return ret.root.text

    def _get_databases(self):
        if self._databases == None:
            ret = self.request("/MassSpecAPI.asmx/GetDatabases?")
            print ret
            self._databases = [x.text for x in ret.getchildren()]
            print self._databases
        return self._databases
    databases = property(_get_databases, doc="Returns databases searched for in chemSpider")




from bioservices import chemspider


def test_chemspider():
    s = chemspider.ChemSpider()
    assert s.find("Pyridine") == [1020]
    s.GetExtendedCompoundInfo(1020)
    s.mol(1020)
    s.mol3d(1020)
    s.image(1020)
    s.ImagesHandler(1020)
    s.databases



# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s): 
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: kegg.py 156 2013-02-17 22:45:39Z cokelaer $
"""Interface to the EUtils web Service.

.. topic:: What is EUtils ?

    :STATUS: in progress. Some functionalities are implemented but not completed yet.

    :URL: http://www.ncbi.nlm.nih.gov/books/NBK25499/#chapter4.EFetch
    :URL: http://www.ncbi.nlm.nih.gov/books/NBK25500/#chapter1.Demonstration_Programs
    :WSDL: http://www.ncbi.nlm.nih.gov/entrez/query/static/esoap_help.html

    .. highlights::

        The Entrez Programming Utilities (E-utilities) are a set of eight server-side programs that provide a stable interface into the Entrez query and database system at the National Center for Biotechnology Information (NCBI). The E-utilities use a fixed URL syntax that translates a standard set of input parameters into the values necessary for various NCBI software components to search for and retrieve the requested data. The E-utilities are therefore the structured interface to the Entrez system, which currently includes 38 databases covering a variety of biomedical data, including nucleotide and protein sequences, gene records, three-dimensional molecular structures, and the biomedical literature. 

       -- from http://www.ncbi.nlm.nih.gov/books/NBK25497/, March 2013

"""

from bioservices import WSDLService
from bioservices import RESTService


__all__ = ["EUtils"]

# Although the wsdl are different, the efetch address can be used indiferently
# so efetch_taxon can be used to request any sql
class EFetch(WSDLService):
    def __init__(self,verbose=False):
        url = "http://www.ncbi.nlm.nih.gov/entrez/eutils/soap/v2.0/efetch_taxon.wsdl"
        super(EFetch, self).__init__(name="EUtilsTaxon", verbose=verbose, url=url)

    def efetch(self, db, Id):
        ret = self.serv.run_eFetch(db=db, id=Id)
        return ret

class EUtilsPubmed(WSDLService):
    def __init__(self,verbose=False):
        url = "http://www.ncbi.nlm.nih.gov/entrez/eutils/soap/v2.0/efetch_taxon.wsdl"
        super(EUtilsPubmed, self).__init__(name="EUtilsTaxon", verbose=verbose, url=url)

    def run_eFetch(self, db, Id):
        ret = self.serv.run_eFetch(db=db, id=Id)
        return ret


class EUtilsTaxon(WSDLService):
    def __init__(self,verbose=False):
        url = "http://www.ncbi.nlm.nih.gov/entrez/eutils/soap/v2.0/efetch_taxon.wsdl"
        super(EUtilsTaxon, self).__init__(name="EUtilsTaxon", verbose=verbose, url=url)

    def run_eFetch(self, db, Id):
        """run_eFetch 

        :param str db:
        :param str Id:
        """
        ret = self.serv.run_eFetch(db=db, id=Id)
        return ret

class EUtilsSNP(WSDLService):
    """

    ret = e.serv.run_eSummary(db="snp",id="7535")
    """
    def __init__(self, verbose=False):
        url = "http://www.ncbi.nlm.nih.gov/entrez/eutils/soap/v2.0/efetch_snp.wsdl"
        super(EUtilsSNP, self).__init__(name="EUtilsSNP", verbose=verbose, url=url)

    def run_eFetch(self, Id):
        ret = self.serv.run_eFetch(id=Id)
        return ret

 
class EUtils(WSDLService):
    """Interface to `NCBI Entrez Utilities <http://www.ncbi.nlm.nih.gov/entrez/query/static/esoap_help.html>`_ service


    This class is not completed. However, you can already access to most of the
    E-utilities. What may be missing is optional keywords to EFecth and some
    other functions. The EPost is not implemented.

    .. warning:: Read the `guidelines
        <http://www.ncbi.nlm.nih.gov/books/NBK25497/>`_ before sending requests.
        No more than 3 requests per seconds otherwise your IP may be banned.

    Here is an example on how to use EFetch to retrieve the FASTA sequence of a
    given ID (34577063)::

        >>> from bioservices import EUtils
        >>> s = EUtils()
        >>> s.EFetch("sequences", "34577063", retmode="text", rettype="fasta")
        >gi|34577063|ref|NP_001117.2| adenylosuccinate synthetase isozyme 2 [Homo sapiens]
        MAFAETYPAASSLPNGDCGRPRARPGGNRVTVVLGAQWGDEGKGKVVDLLAQDADIVCRCQGGNNAGHTV
        VVDSVEYDFHLLPSGIINPNVTAFIGNGVVIHLPGLFEEAEKNVQKGKGLEGWEKRLIISDRAHIVFDFH
        QAADGIQEQQRQEQAGKNLGTTKKGIGPVYSSKAARSGLRMCDLVSDFDGFSERFKVLANQYKSIYPTLE
        IDIEGELQKLKGYMEKIKPMVRDGVYFLYEALHGPPKKILVEGANAALLDIDFGTYPFVTSSNCTVGGVC
        TGLGMPPQNVGEVYGVVKAYTTRVGIGAFPTEQDNEIGELLQTRGREFGVTTGRKRRCGWLDLVLLKYAH
        MINGFTALALTKLDILDMFTEIKVGVAYKLDGEIIPHIPANQEVLNKVEVQYKTLPGWNTDISNARAFKE
        LPVNAQNYVRFIEDELQIPVKWIGVGKSRESMIQLF

    Most of the functions takes a database name as input. You can obtain the
    valid list by checking the :attr:`databases` attribute.

    A few functions takes Identifier(s) as input. It must be a string of one Id
    or several Ids, in which case they must be comma sperated without spaces::

        Incorrect: id=352, 25125, 234
        Correct:   id=352,25125,234

    A few functions takes an argument called **term**. You can use the **AND**
    keyword but spaces must be replaced by + signs::

        Incorrect: term=biomol mrna[properties] AND mouse[organism]
        Correct:   term=biomol+mrna[properties]+AND+mouse[organism]

    Other special characters, such as quotation marks (") or the # symbol used
    in referring to a query key on the History server, should be represented by
    their URL encodings (%22 for "; %23 for #).::

        Incorrect: term=#2+AND+"gene in genomic"[properties]
        Correct:   term=%232+AND+%22gene+in+genomic%22[properties]

    """
    def __init__(self, verbose=False):
        url = "http://www.ncbi.nlm.nih.gov/entrez/eutils/soap/v2.0/eutils.wsdl?"
        super(EUtils, self).__init__(name="EUtils", verbose=verbose, url=url)


        self._databases = None
        # some extra links
        self._efetch_taxonomy = EUtilsTaxon(verbose=verbose)
        self._efetch_snp = EUtilsSNP(verbose=verbose)
        self._efetch = EFetch(verbose=verbose)
        self.tool = "bioservices"   # not used but may be used later on 
        self.email = "unknown"      # not used but may be used later on

    def _get_databases(self):
        """alias to run_eInfo"""
        if self._databases == None:
            self._databases = self.serv.run_eInfo().DbName
        return self._databases
    databases = property(_get_databases, doc="Returns list of valid databases")

    def taxonomy(self, Id):
        """Alias to EFetch for ther taxonomy database using WSDL

        ::

            >>> s = EUtils()
            >>> ret = s.taxonomy("9606")
            >>> ret.Taxon.TaxId
            '9606'
            >>> ret.Taxon.ScientificName
            'Homo sapiens'
        """
        ret = self._efetch_taxonomy.serv.run_eFetch(db="taxonomy", id=Id)
        return ret 

    def snp(self, Id):
        """Alias to Efetch for the SNP database using WSDL

        ::

            >>> s.snp("123")

        """
        ret = self._efetch_snp.run_eFetch(Id)
        return ret

    def EFetch(self, db, Ids, retmode="xml", **kargs):
        """Access to the EFetch E-Utilities

        :param str db: Database from which to retrieve UIDs. The value must be a valid Entrez database
            name . This is the destination database for the link operation. 
        :param str Ids: UID list. Either a single UID or a comma-delimited list of UIDs may be provided.
            All of the UIDs must be from the database specified by db. Limited  to 200 Ids

        ::

            >>> ret = s.EFetch("omim", "269840")  --> ZAP70
            
            >>> ret = s.EFetch("taxonomy", "9606")
            >>> [x.text for x in ret.getchildren()[0].getchildren() if x.tag=="ScientificName"]
            ['Homo sapiens']

            >>> s = eutils.EUtils()
            >>> s.EFetch("sequences", "34577063", retmode="text", rettype="fasta")
            >gi|34577063|ref|NP_001117.2| adenylosuccinate synthetase isozyme 2 [Homo sapiens]
            MAFAETYPAASSLPNGDCGRPRARPGGNRVTVVLGAQWGDEGKGKVVDLLAQDADIVCRCQGGNNAGHTV
            VVDSVEYDFHLLPSGIINPNVTAFIGNGVVIHLPGLFEEAEKNVQKGKGLEGWEKRLIISDRAHIVFDFH
            QAADGIQEQQRQEQAGKNLGTTKKGIGPVYSSKAARSGLRMCDLVSDFDGFSERFKVLANQYKSIYPTLE
            IDIEGELQKLKGYMEKIKPMVRDGVYFLYEALHGPPKKILVEGANAALLDIDFGTYPFVTSSNCTVGGVC
            TGLGMPPQNVGEVYGVVKAYTTRVGIGAFPTEQDNEIGELLQTRGREFGVTTGRKRRCGWLDLVLLKYAH
            MINGFTALALTKLDILDMFTEIKVGVAYKLDGEIIPHIPANQEVLNKVEVQYKTLPGWNTDISNARAFKE
            LPVNAQNYVRFIEDELQIPVKWIGVGKSRESMIQLF

        The alias :meth:`taxonomy` uses the WSDL and provides an easier way to manipulate
        the ouptut.

        .. note:: uses the REST service only

        .. todo:: more documentation and optional arguments
        """
        #self._check_db(db)
        self._check_retmode(retmode)
        self._check_ids(Ids)
        s = RESTService("test","http://eutils.ncbi.nlm.nih.gov/entrez/eutils/")
        request = "efetch.fcgi?db=%s&id=%s&retmode=%s" % (db, Ids, retmode)

        if kargs.get("strand"):
            strand = kargs.get("strand")
            if strand in [1,2]:
                request += "&strand=%s" % strand
            else:
                raise ValueError("invalid strand. must be a number in 1,2")

        if kargs.get("complexity"):
            strand = kargs.get("complexity")
            if strand in [0,1,2,3,4]:
                request += "&complexity=%s" % strand
            else:
                raise ValueError("invalid complexity. must be a number in 0,1,2,3,4")

        if kargs.get("seq_start"):
            request += "&seq_start=%s" % kargs.get("seq_start")
        if kargs.get("seq_stop"):
            request += "&seq_stop=%s" % kargs.get("seq_stop")

        if kargs.get("rettype"):
            print kargs.get("rettype")
            request += "&rettype=%s" % kargs.get("rettype")

        ret = s.request(request)

        return ret


    def EInfo(self, db=None, method="wsdl"):
        """Provides the number of records indexed in each field of a given
        database, the date of the last update of the database, and the available links
        from the database to other Entrez databases.

        :param str db: target database about which to gather statistics. Value must be a
            valid Entrez database name. See :attr:`databases` or don't provide
            db to get the list
        :return: statistics with XML format if method is "rest") or SOAP format
            if method is wsdl (default).

        ::

            >>> ret = s.EInfo("taxonomy")
            >>> ret.Count

        If method is xml, the output is in BeautifulSoup format::

            >>> ret = s.EInfo("taxonomy", method="rest")

        """
        if db == None:
            return self.databases
        else:
            self._check_db(db)

        if method=="wsdl":
            ret = self._einfo_wsdl(db)
        elif method == "rest":
            ret = self._einfo_rest(db)
        else:
            raise ValueError("method must be either wsdl or rest")
        return ret

    def _einfo_rest(self, db=None):
        s = RESTService("test","http://eutils.ncbi.nlm.nih.gov/entrez/eutils/")
        ret = s.request("einfo.fcgi?db=%s" % db)
        return ret

    def _einfo_wsdl(self, db=None):
        return self.serv.run_eInfo(db=db) 

    def _check_ids(self, Ids):
        if len(Ids.split(","))>200:
            raise ValueError("Number of comma separated IDs must be less than 200")

    def _check_db(self, db):
        if db not in self.databases:
            raise ValueError("You must provide a valid databases from : ", self.databases)

    def _check_retmode(self, retmode):
        if retmode not in ["xml", "text"]:
            raise ValueError("You must provide a retmode in 'xml', 'test'")

    def ESummary(self, db, Ids, method="wsdl"):
        """document summary downloads 

        Returns document summaries (DocSums) for a list of input UIDs
        OR returns DocSums for a set of UIDs stored on the Entrez History server

        :param str Ids: UID list. Either a single UID or a comma-delimited list of UIDs may be provided.
            All of the UIDs must be from the database specified by db. Limited  to 200 Ids

        ::

            >>> from bioservices import *
            >>> s = EUtils()
            >>> ret = s.ESummary("snp","7535")
            >>> ret = s.ESummary("snp","7535,7530")
            >>> ret = s.ESummary("taxonomy", "9606,9913")

        .. warning:: the query_key, WebEnv, restart, retmax, and version options
            are not yet implemented.
        """
        self._check_ids(Ids)
        if db == None:
            return self.databases
        else:
            self._check_db(db)

        if method=="wsdl":
            ret = self._esummary_wsdl(db, Ids)
        elif method == "rest":
            ret = self._esummary_rest(db, Ids)
        else:
            raise ValueError("method must be either wsdl or rest")
        return ret

    def _esummary_rest(self, db, Ids):
        # [(x.attrib['Name'], x.text) for x in ret.getchildren()[0].getchildren()[1:]]
        s = RESTService("test","http://eutils.ncbi.nlm.nih.gov/entrez/eutils/")
        ret = s.request("esummary.fcgi?db=%s&id=%s" % (db, Ids))
        return ret

    def _esummary_wsdl(self, db, Ids):
        ret = self.serv.run_eSummary(db=db, id=Ids)
        return ret

    def EGquery(self, term, retmode="xml", method="wsdl"):
        """Provides the number of records retrieved in all Entrez databases by a single text query.

        :param str term: Entrez text query. All special characters must be URL
            encoded. Spaces may be replaced by '+' signs. For very long queries (more than
            several hundred characters long), consider using an HTTP POST call. See the
            PubMed or Entrez help for information about search field descriptions and tags.
            Search fields and tags are database specific.

        ::

            >>> ret = s.EGquery("asthma")
            >>> [(x.DbName, x.Count) for x in ret.eGQueryResult.ResultItem if x.Count!='0']

        .. note:: WSDL (default) and REST protocol available
        .. todo:: documentation and optional arguments
        """
        self._check_retmode(retmode)
        if method == "wsdl":
            ret = self._egquery_wsdl(term, retmode)
        elif method == "rest":
            ret = self._egquery_rest(term, retmode)
        else:
            raise ValueError("method must be either wsdl or rest")
        return ret

    def _egquery_rest(self, term, retmode="xml"):
        self._check_retmode(retmode)
        s = RESTService("test","http://eutils.ncbi.nlm.nih.gov/entrez/eutils/")
        ret = s.request("egquery.fcgi?term=%s&retmode=%s" % (term, retmode))
        return ret

    def _egquery_wsdl(self, term, retmode="xml"):
        """retmode does not seem to work in the wsdl case"""
        self._check_retmode(retmode)
        if retmode == "xml":
            ret = self.serv.run_eGquery(term=term, retmode="xml")
        else:
            ret = self.serv.run_eGquery(term=term)
        return ret


    def ESpell(self, db, term):
        """Provides spelling suggestions for terms within a single text query in a given database.

        :param str db: database to search. Value must be a valid Entrez database name (default = pubmed).
        :param str term: Entrez text query. All special characters must be URL encoded. Spaces may be replaced by '+' signs. For very long queries (more than several hundred characters long), consider using an HTTP POST call. See the PubMed or Entrez help for information about search field descriptions and tags. Search fields and tags are database specific.


        ::

            >>> ret = e.ESpell(db="omim", term="aasthma+OR+alergy")
            >>> ret.Query
            'asthmaa OR alergies'
            >>> ret.CorrectedQuery
            'asthma or allergy'

        .. note:: only WSDL protocol available 
        """
        self._check_db(db)
        ret = self.serv.run_eSpell(db=db, term=term)
        return ret


    def ELink(self, db, dbfrom, Ids, cmd="neighbor"):
        """The Entrez links utility

        Responds to a list of UIDs in a given database with either a list of
        related UIDs (and relevancy scores) in the same database or a list of linked
        UIDs in another Entrez database; checks for the existence of a specified link
        from a list of one or more UIDs; creates a hyperlink to the primary LinkOut
        provider for a specific UID and database, or lists LinkOut URLs and attributes
        for multiple UIDs.

        :param str db: Database from which to retrieve UIDs. The value must be a valid Entrez database
            name. This is the destination database for the link operation.
        :param str dbfrom: Database containing the input UIDs. The value must be a
            valid Entrez database name (default = pubmed). This is the origin database of
            the link operation. If db and dbfrom are set to the same database value, then
            ELink will return computational neighbors within that database. Please see the
            full list of Entrez links for available computational neighbors. Computational
            neighbors have linknames that begin with dbname_dbname (examples:
            protein_protein, pcassay_pcassay_activityneighbor).
        :param str Ids: UID list. Either a single UID or a comma-delimited list of UIDs may be provided.
            All of the UIDs must be from the database specified by db. Limited  to 200 Ids
        :param str cmd: ELink command mode. The command mode specified which
            function ELink will perform. Some optional parameters only function for certain
            values of cmd (see http://www.ncbi.nlm.nih.gov/books/NBK25499/#chapter4.ELink).
            Examples are neighbor, prlinks.

        :: 

            >>> # Example: Find related articles to PMID 20210808
            >>> ret = s.ELink("pubmed", "pubmed", Ids="20210808", cmd="neighbor_score")


        """
        self._check_ids(Ids)
        self._check_db(db)
        self._check_db(dbfrom)
        assert cmd in ["neighbor", "neighbor_score", "neighbor_history",
"acheck", "llinks", "lcheck", "ncheck", "llinkslib" ,"prlinks"]

        s = RESTService("test","http://eutils.ncbi.nlm.nih.gov/entrez/eutils/")

        request = "elink.fcgi?db=%s&dbfrom=%s" % (db, dbfrom)
        request += "&id=%s" % Ids
        request += "&cmd=%s" % cmd
        ret = s.request(request)
        return ret

    def EPost(self):
        """Not implemented.

        """
        raise NotImplementedError




# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s): 
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: uniprot.py 172 2013-03-21 16:22:28Z cokelaer $
"""Interface to some part of the UniProt web service

.. topic:: What is UniProt ?

    :URL: http://www.uniprot.org
    :Citation:

    .. highlights::

        "The Universal Protein Resource (UniProt) is a comprehensive resource for protein
        sequence and annotation data. The UniProt databases are the UniProt
        Knowledgebase (UniProtKB), the UniProt Reference Clusters (UniRef), and the
        UniProt Archive (UniParc). The UniProt Metagenomic and Environmental Sequences
        (UniMES) database is a repository specifically developed for metagenomic and
        environmental data."

        -- From Uniprot web site (help/about) , Dec 2012


.. mapping betwenn uniprot and bench of other DBs.
.. ftp://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/idmapping/


"""
from services import Service, RESTService
import urllib2

__all__ = ["UniProt"]


mapping = {"UniProtKB AC/ID":"ACC+ID", 
    "UniProtKB": "AC",
    "UniProtKB": "ID",
    "UniParc": "UPARC",
    "UniRef50": "NF50",
    "UniRef90": "NF90",
    "UniRef100": "NF100",
    "EMBL/GenBank/DDBJ": "EMBL_ID",
    "EMBL/GenBank/DDBJ CDS": "EMBL",
    "PIR": "PIR",
    "UniGene": "UNIGENE_ID",
    "Entrez Gene (GeneID)": "P_ENTREZGENEID",
    "GI number*":"P_GI", 
    "IPI": "P_IPI",
    "RefSeq Protein": "P_REFSEQ_AC",
    "RefSeq Nucleotide": "REFSEQ_NT_ID",
    "PDB": "PDB_ID",
    "DisProt": "DISPROT_ID",
    "HSSP": "HSSP_ID",
    "DIP": "DIP_ID",
    "MINT": "MINT_ID",
    "Allergome": "ALLERGOME_ID",
    "MEROPS": "MEROPS_ID",
    "mycoCLAP": "MYCOCLAP_ID",
    "PeroxiBase": "PEROXIBASE_ID",
    "PptaseDB": "PPTASEDB_ID",
    "REBASE": "REBASE_ID",
    "TCDB": "TCDB_ID",
    "PhosSite": "PHOSSITE_ID",
    "DMDM": "DMDM_ID",
    "Aarhus/Ghent-2DPAGE": "AARHUS_GHENT_2DPAGE_ID",
    "World-2DPAGE": "WORLD_2DPAGE_ID",
    "DNASU": "DNASU_ID",
    "Ensembl": "ENSEMBL_ID",
    "Ensembl Protein": "ENSEMBL_PRO_ID",
    "Ensembl Transcript": "ENSEMBL_TRS_ID",
    "Ensembl Genomes": "ENSEMBLGENOME_ID",
    "Ensembl Genomes Protein": "ENSEMBLGENOME_PRO_ID",
    "Ensembl Genomes Transcript": "ENSEMBLGENOME_TRS_ID",
    "GeneID": "P_ENTREZGENEID",
    "GenomeReviews": "GENOMEREVIEWS_ID",
    "KEGG": "KEGG_ID",
    "PATRIC": "PATRIC_ID",
    "UCSC": "UCSC_ID",
    "VectorBase": "VECTORBASE_ID",
    "AGD": "AGD_ID",
    "ArachnoServer": "ARACHNOSERVER_ID",
    "CGD": "CGD",
    "ConoServer": "CONOSERVER_ID",
    "CYGD": "CYGD_ID",
    "dictyBase": "DICTYBASE_ID",
    "EchoBASE": "ECHOBASE_ID",
    "EcoGene": "ECOGENE_ID",
    "euHCVdb": "EUHCVDB_ID",
    "EuPathDB": "EUPATHDB_ID",
    "FlyBase": "FLYBASE_ID",
    "GeneCards": "GENECARDS_ID",
    "GeneFarm": "GENEFARM_ID",
    "GenoList": "GENOLIST_ID",
    "H-InvDB": "H_INVDB_ID",
    "HGNC": "HGNC_ID",
    "HPA": "HPA_ID",
    "LegioList": "LEGIOLIST_ID",
    "Leproma": "LEPROMA_ID",
    "MaizeGDB": "MAIZEGDB_ID",
    "MIM": "MIM_ID",
    "MGI": "MGI_ID",
    "neXtProt": "NEXTPROT_ID",
    "Orphanet": "ORPHANET_ID",
    "PharmGKB": "PHARMGKB_ID",
    "PomBase": "POMBASE_ID",
    "PseudoCAP": "PSEUDOCAP_ID",
    "RGD": "RGD_ID",
    "SGD": "SGD_ID",
    "TAIR": "TAIR_ID",
    "TubercuList": "TUBERCULIST_ID",
    "WormBase": "WORMBASE_ID",
    "WormBase Transcript": "WORMBASE_TRS_ID",
    "WormBase Protein": "WORMBASE_PRO_ID",
    "Xenbase": "XENBASE_ID",
    "ZFIN": "ZFIN_ID",
    "eggNOG": "EGGNOG_ID",
    "GeneTree": "GENETREE_ID",
    "HOGENOM": "HOGENOM_ID",
    "HOVERGEN": "HOVERGEN_ID",
    "KO": "KO_ID",
    "OMA": "OMA_ID",
    "OrthoDB": "ORTHODB_ID",
    "ProtClustDB": "PROTCLUSTDB_ID",
    "BioCyc": "BIOCYC_ID",
    "Reactome": "REACTOME_ID",
    "UniPathWay": "UNIPATHWAY_ID",
    "CleanEx": "CLEANEX_ID",
    "GermOnline": "GERMONLINE_ID",
    "ChEMBL": "CHEMBL_ID",
    "ChiTaRS": "CHITARS_ID",
    "DrugBank": "DRUGBANK_ID",
    "GenomeRNAi": "GENOMERNAI_ID",
    "NextBio": "NEXTBIO_ID"
}




class UniProt(RESTService):
    """Interface to the `UniProt <http://www.uniprot.org>`_ service

    .. warning:: for the time being, this class only provide interface to 
        * the identifier mapping service
        * search of a UniProtKB identifier
        * some experimental interface to the full search 

    Example::

        >>> from bioservices import UniProt
        >>> u = UniProt(verbose=False)
        >>> u.mapping(fr="ACC", to="KEGG_ID", query='P43403')
        ['FromACC', 'ToKEGG_ID', 'P43403', 'hsa:7535']
        >>> res = u.search("P43403")

        # Returns sequence on the ZAP70_HUMAN accession Id
        >>> sequence = u.search("ZAP70_HUMAN", 
            format="tab", columns="sequence")

    """
    _mapping = mapping.copy()
    _url = "http://www.uniprot.org"
    def __init__(self, verbose=True):
        """**Constructor**

        :param verbose: set to False to prevent informative messages
        """
        super(UniProt, self).__init__(name="UniProt", url=UniProt._url, verbose=verbose)


    def mapping(self, fr="ID", to="KEGG_ID", format="tab", query="P13368"):
        """This is an interface to the UniProt mapping service


        ::

            >>> u.mapping(fr="ACC", to="KEGG_ID", query='P43403')
            ['From:ACC', 'To:KEGG_ID', 'P43403', 'hsa:7535']
            >>> u.mapping(fr="ACC", to="KEGG_ID", query='P43403 P00958')
            ['From:ACC', 'To:KEGG_ID', 'P43403', 'hsa:7535', 'P00958', 'sce:YGR264C']


        There is a web page that gives the list of correct `database identifiers
        <http://www.uniprot.org/faq/28>`_

        :URL: http://www.uniprot.org/mapping/

        """
        import urllib
        url = self.url + '/mapping/'
        params = {'from':fr, 'to':to, 'format':format, 'query':query}
        data = urllib.urlencode(params)
        self.logging.info(data)
        request = urllib2.Request(url, data)
        # 2 following lines are optional
        contact = ""
        request.add_header('User-Agent', 'Python contact')
        response = urllib2.urlopen(request)
        result = response.read(200000)

        # let us improvve the output a little bit using a list  instead of a
        # string
        try:
            result = result.split()
            result[0]+=':'+fr
            result[1]+=':'+to
        except:
            pass

        return result

    def searchUniProtId(self, uniprot_id, format="xml"):
        """Search for a uniprot ID in UniprotKB database

        :param str uniprot: a valid uniprotKB ID
        :param str format: expected output format amongst xml, txt, fasta, gff, rdf


        ::

            >>> u = UniProt()
            >>> res = u.searchUniProtId("P09958", format="xml")

        """
        _valid_formats = ['txt', 'xml', 'rdf', 'gff', 'fasta']
        self.checkParam(format, _valid_formats)
        #if format not in _valid_formats:
        #    raise ValueError("invalid format provided. Use one of %s" % _valid_formats)
        url = self.url + "/uniprot/" + uniprot_id + '.' + format
        res = self.request(url)
        return res


    def search(self, query, format="html", columns=None,
        include=False,sort="score", compress=False, limit=None, offset=None, maxTrials=10):
        """Provide some interface to the uniprot search interface.

        :param str query: query must be a valid uniprot query.
            See http://www.uniprot.org/help/text-search, http://www.uniprot.org/help/query-fields 
        :param str format: a valid format amongst html, tab, xls, asta, gff,
            txt, xml, rdf, list, rss. If tab or xls, you can also provide the 
            columns argument. 
        :param str columns: comma-separated list of values. Works only if fomat 
            is tab or xlsFor UnitProtKB, the possible columns are:
            citation, clusters, comments, database, domains, domain, ec, id, entry name
            existence, families, features, genes, go, go-id, interpro, interactor,
            keywords, keyword-id, last-modified, length, organism, organism-id, 
            pathway, protein names, reviewed, score, sequence, 3d, 
            subcellular locations, taxon, tools, version, virus hosts
        :param bool include:  include isoform sequences when the format
            parameter is fasta. Include description when format is rdf. 
        :param sort: by score by default. Set to None to bypass this behaviour
        :param bool compress: gzip the results
        :param int limit: Maximum number of results to retrieve.
        :param int offset:  Offset of the first result, typically used together
            with the limit parameter.
        :param int maxTrials: this request is unstable, so we may want to try
            several time.

        To obtain the list of uniprot ID returned by the search of zap70 can be retrieved as follows
        ::

            >>> u.search('zap70+AND+organism:9606', format='list')
            >>> u.search("zap70+and+taxonomy:9606", format="tab", limit=3, 
            ...    columns="entry name,length,id, genes")
            Entry name  Length  Entry   Gene names
            CBLB_HUMAN  982 Q13191  CBLB RNF56 Nbla00127
            CBL_HUMAN   906 P22681  CBL CBL2 RNF55
            CD3Z_HUMAN  164 P20963  CD247 CD3Z T3Z TCRZ



        .. warning:: this function request seems a bit unstable (UniProt web issue ?)
            so we repeat the request if it fails
        """
        params = {}

        if format!=None:
            _valid_formats = ['tab', 'xls', 'fasta', 'gff', 'txt', 'xml', 'rss', 'list', 'rss', 'html']
            self.checkParam(format, _valid_formats)
            params['format'] = format

        if columns!=None:
            self.checkParam(format, ["tab","xls"])
            _valid_columns = ['citation', 'clusters', 'comments','database',
                'domains','domain', 'ec','id','entry name','existence',
        		'families', 'features', 'genes', 'go', 'go-id', 'interpro', 
                'interactor', 'keywords', 'keyword-id', 'last-modified', 
                'length', 'organism', 'organism-id', 'pathway', 'protein names', 
                'reviewed', 'score', 'sequence', '3d', 'subcellular locations', 
                'taxonomy', 'tools', 'version', 'virus hosts']
            # remove unneeded spaces before/after commas if any
            if "," in columns:
                columns = [x.strip() for x in columns.split(",")]
            else:
                columns = [columns]
            for col in columns:
                self.checkParam(col, _valid_columns)

            # convert back to a string as expected by uniprot
            params['columns'] = ",".join([x.strip() for x in columns])

        if include == True and format in ["fasta", "rdf"]:
            params['include'] = 'yes'

        if compress == True:
            params['compress'] = 'yes'
 
        if sort:
            self.checkParam(sort, ["score"])
            params['sort'] = sort

        if offset != None:
            if isinstance(offset, int):
                params['offset'] = offset

        if limit != None:
            if isinstance(limit, int):
                params['limit'] = limit

        params = self.urlencode(params)
        
        #res = s.request("/uniprot/?query=zap70+AND+organism:9606&format=xml", params)
        trials = 1
        while trials<maxTrials:
            try:
                res = self.request("uniprot/?query=%s" % query + "&" + params, "txt")
                trials = maxTrials + 1
            except:
                self.logging.warning("Trying again...")
                import time
                time.sleep(2)
                trials += 1
        return res



    def quick_search(self, query, include=False,sort="score", limit=None):

        res = self.search(query, "tab", include=include, sort=sort, limit=limit)

        #if empty result, nothing to do
        if len(res) == 0:
            return res
        # else populate a dictionary
        newres = {}
        for line in res.split("\n")[1:-1]:
            print line
            Entry, a,b,c,d,e,f = line.split("\t")
            print Entry, a, b, c, d, e, f
            newres[Entry] = { 'Entry name': a,
                         'Status': b,
                         'Protein names': c,
                         'Gene names': d,
                         'Organism' : e,
                         'Length' : f}
        return newres



# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s):
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: $
"""Interface to the WikiPathway service

.. topic:: What is WikiPathway ?


    :URL: http://www.wikipathways.org/index.php/WikiPathways
    :WSDL: http://www.wikipathways.org/index.php/Help:WikiPathways_Webservice/API
    :Citation: `doi:10.1371/journal.pone.0006447 <http://www.plosone.org/article/info:doi/10.1371/journal.pone.0006447>`_

    .. highlights::

        " WikiPathways is an open, public platform dedicated to the curation of
        biological pathways by and for the scientific community."

        -- From WikiPathway web site. Dec 2012



"""
from services import WSDLService, RESTService
import copy, webbrowser, xmltools, base64


__all__ = ["Wikipathway"]


# this is useless but let us keep for now
class _Items2(object):
    def __init__(self, data, name = 'item', verbose = True):
        self.data = data

        self._items = copy.deepcopy(data)
        self.definitions = self._items  ## Consider replace self_items with this variable
#    def _get_entry_ids(self):
#    def _get_definitions(self):
#        ids = [x[

    def _get_entry_ids(self):
        ids = [x['id'] for x in self.items]
        return ids
    entry_ids = property(_get_entry_ids, doc = "return list of ids")

    def _get_names(self):
        name = [x['name'] for x in self.items]
        return name
    name = property(_get_names, doc = "return list of names")

    def _get_items(self):
        return self._items
    items = property(_get_items,doc=_get_items.__doc__)
    #def _get_entry_ids(self):


class Wikipathway(WSDLService):
    """Interface to `Pathway <http://www.wikipathways.org/index.php>`_ service

    .. doctest::

       >>> from bioservices import *
       >>> s = Wikipathway()
       >>> s.organism  # default organism
       'Homo sapiens'

    Examples::

        s.findPathwaysByText('MTOR')
        s.getPathway('WP1471')
        s.getPathwaysByOntologyTerm('DOID:344')
        s.findPathwaysByXref('P45985')

    The methods that require a login are not implemented (:meth:`login`,
    :meth:`updatePathway`, :meth:`removeCurationTag`, :meth:`saveCurationTag`,
    :meth:`createPathway`)

    Methods not implemented at all:

      * getXrefList: Neither WSDL or REST seemed to work
      * u'getCurationTagHistory': No API found in Wikipathway web page
      * u'getRelations': No API found in WikiPathway web page
    """
    _url = 'http://www.wikipathways.org/wpi/webservice/webservice.php?wsdl'
    def __init__(self, verbose = True):
        """.. rubric:: Constructor

        :param bool verbose:

        """

        super(Wikipathway, self).__init__(name="Wikipathway", url=Wikipathway._url, verbose = verbose)
        self._organism = 'Homo sapiens' ## This function is redundant (see class service)
        self.logging.info("Fetching organisms...")

        #: Get a list of all available organisms.
        self.organisms = self.serv.listOrganisms()

#        self.recentChanges = Items2(self.serv.getRecentChanges(time

    def _set_organism(self, organism):
        if organism in self.organisms:
            self._organism = organism
        else:
            raise ValueError("'%s' is not supported in Wikipathways. See :attr:`organisms`" % organism)

    def _get_organism(self):
        return self._organism
    organism = property(_get_organism, _set_organism, doc = "Read/write attribute for the organism")

    # REST /findPathwaysByLiterature?query=18651794
    def findPathwaysByLiterature(self, query):
        """Find pathways by their literature references.

        :param str query: The query, can be a pubmed id, author name or title keyword.
        :return:  Array of WSSearchResult descr=The search results. {{{descr}}}

        ::

            s.findPathwaysByLiterature(18651794)
        """
        res = self.serv.findPathwaysByLiterature(query=query)
        return res

    def findPathwaysByXref(self, ids):
        """Find pathways by searching on the external references of DataNodes.


        This function supports only 1 ids and 1 code at a time. To specify 
        multiple ids and codes parameters to query for multiple xrefs at once,
        the REST syntax should be used. In that case, the number of ids and 
        codes parameters should match, they will be paired to form xrefs, 
        e.g.::

            from bioservices import RESTService
            r = RESTService("test", url=s.url[:-5])
            r.request('/findPathwaysByXref?ids=1234&ids=ENSG00000130164&codes=L&codes=EnHs')
            r.request('/findPathwaysByXref?ids=1234&codes=L')

        :param str string ids: One DataNode identifier(s) (e.g. 'P45985').
            Datanodes can be (gene/protein/metabolite identifiers). 
        :param str codes: One code of the database system to limit the search
            to. **Not implemented**.
        :return:  List of WSSearchResult. An array of search results with DataNode GraphId stored in the 'field' hash.


        ::

            s.findPathwaysByXref(ids="P45985")


        .. warning:: **codes** is not available. Does not seem to work in WSDLinterface.
        """
        # codes=code but does not seem to work
        res = self.serv.findPathwaysByXref(ids=ids)
        return res

    #REST: s.url[:-5] + ?query=P53
    def findInteractions(self, query, organism=None,interactionOnly=True, 
        raw = False):
        """Find interactions defined in WikiPathways pathways.


        :param str query:  The name of an entity to find interactions for (e.g. 'P53')
        :param str organism:  The name of the organism to refine the search
            (default is the :attr: 'organism' attribute).
        :param bool interactionOnly: Returns only the interactions (default). If
            false, returns also scores, revision, pathways
        :param bool raw: If True, returns the output of the request without post processing (also ignoring organism)
        :returns: Depends on the parameters **raw** and **interactionOnly**. By default, 
            the output from WikiPathways is processed and only the interactions
            are returned (for the default organism). You can change this behaviour
            by changing the default arguments (raw, organism, interactionOnly)

        .. warning:: Interface different from the service, unless raw is set to True

        """
        if raw:
            return self.serv.findInteractions(query=query)
        else:
            output = {'interactions':[],'scores':[],'pathway_ids':[],'revision':[],}
            intA = []
            intB = []
            if organism == None:
                organism = self.organism
            for x in self.serv.findInteractions(query=query):
                if 'species' not in x._keyord:
                    continue
                if x['species'] == organism:
                    intA.append(x['fields'][1]['values'])
                    intB.append(x['fields'][2]['values'])
                    output['scores'].append(x['score'])
                    output['pathway_ids'].append(x['id'])
                    output['revision'].append(x['revision'])
            output['interactions'] = zip(intA,intB)
            if interactionOnly==False:
                return output
            else:
                return output['interactions']

    def listPathways(self, organism = None):
        """Get a list of all available pathways.

        :param str organism: a valid organism (default is the :attr:`organism` attribute)
        :return: List of pathways for the selected organism.

        """
        if organism == None:
            return self.serv.listPathways(organism = self.organism)
        else:
            self.checkParam(organism, self.organisms)
            return self.serv.listPathways(organism = organism)

    def getPathway(self, pathwayId, revision=0):
        """Download a pathway from WikiPathways.

        :param str pathwayId: the pathway identifier.
        :param int revision: the revision number of the pathway (use '0'
            for most recent version).
        :Returns: The pathway.

        ::

            s.getPathway("WP2320")
        """
        return self.serv.getPathway(pwId=pathwayId, revision=revision)

    def getPathwayInfo(self, pathwayId):
        """Get some general info about the pathway.

        :param str pathwayId: the pathway identifier.
        :return: The pathway info.

        ::

            >>> from bioservices import *
            >>> s= Wikipathway(verbose=False)
            >>> s.getPathwayInfo("WP2320")
        """
        return self.serv.getPathwayInfo(pwId=pathwayId)

    def getPathwayHistory(self, pathwayId, date):
        """Get the revision history of a pathway.

        :param str pathwayId: the pathway identifier.
        :param str date: limit the results by date, only history items after
            the given date (timestamp format) will be included. Can be a string
            or number of the form YYYYMMDDHHMMSS.
        :return: The revision history.

        .. warning:: Does not seem to work with WSDL. Replaced by a REST version but
            unstable: Does not return the results systematically.

        ::

            s.getPathwayHistory("WP4", 20110101000000)
        """
        s = RESTService("WP", url=self.url[:-5], verbose=False)
        res = s.request("getPathwayHistory?pwId=%s&timestamp=%s" % (pathwayId, str(date)))
        return res
        # does not seem to work
        #return self.serv.getPathwayHistory(pwId=pathwayId, timestamp=date)

    def getRecentChanges(self, timestamp):
        """Get the recently changed pathways.

        :param str timestamp: Only get changes from after this time. Timestamp
            format: yyyymmddMMHHSS (string or number)
        :return: The changed pathways

        ::

            s.getRecentChanges(20110101000000)
        """
        return self.serv.getRecentChanges(timestamp=timestamp)

    def login(self, usrname, password):
        """Start a logged in session using an existing WikiPathways account.

        .. warning:: Interface not exposed in bioservices.

        This function will return an authentication code that can
        be used to excecute methods that need authentication (e.g.
        updatePathway).

        :param str name: The username of the WikiPathways account.
        :param str password: The password of the WikiPathways account.


        :Returns: The authentication code for this session.

        """
        raise NotImplementedError
        # for future usage. pass is a python keyword so we must use a dictionary
        #d = {"name":usrname, "pass":password}
        #return self.serv.login(**d)

    def getPathwayAs(self, pathwayId, filetype='owl', revisionNumb=0):
        """Download a pathway in the specified file format.

        :param str pathwayId: the pathway identifier.
        :param str filetype: the file format (default is .owl).
        :param int revision: the revision number of the pathway (use '0' for most recent version - this is default).
        :return: The file contents

        .. warning:: Argument pathwayId and filetype are inversed as compared to the
            WSDL prototype (if you want to call it directly)


        """
        self.checkParam(filetype, ['gpml','png','svg','pdf','txt','pwf', 'owl'])
        res = self.serv.getPathwayAs(fileType = filetype, pwId = pathwayId, 
             revision = revisionNumb)
        return base64.b64decode(res)

    def savePathwayAs(self, pathwayId, filename, revisionNumb = 0, display = True):
        """Save a pathway.

        :param str pathwayId: the pathway identifier.
        :param str filename: the name of the file. If a filename extension is not provided the pathway will be saved as a pdf (default).
        :param int revisionNumb: the revision number of the pathway (use '0 for most recent version).
        :param bool display: if True the pathway will be displayed in your browser.

        .. note:: Method from bioservices. Not a WikiPathways function
        """
        if filename.find('.') == -1:
            filename = "%s.%s" %(filename,'pdf')
        filetype = filename.split('.')[-1]
        res = self.serv.getPathwayAs(fileType = filetype, pwId = pathwayId, revision = revisionNumb)
        f = open(filename,'w')
        f.write(base64.b64decode(res))
        if display:
            webbrowser.open(filename)
        f.close()

    def displaySavedPathwayInBrowser(self, filename):
        """Show a saved document in a browser.

        :param str filename:
        :return: Nothing
        
        .. note:: Method from Bioservices. Not a WikiPathways function.
        """
        webbrowser.open(filename)

    def updatePathway(self, pathwayId, describeChanges, gpmlCode, revision=0):
        """Update a pathway on WikiPathways website with a given GPML code.

        .. warning:: Interface not exposed in bioservices.

        .. note:: To create/modify pathways via the web service, you need to
            have an account with web service write permissions. Please contact
            us to request write access for the web service.

        :param str pwId:  The pathway identifier.
        :param str description:  A description of the modifications.
        :param str gpml: The updated GPML code.
        :param int revision: The revision number of the version this GPML
            code was based on. This is used to prevent edit conflicts in
            case another client edited the pathway after this client downloaded it.
        :param object WSAuth_auth:  The authentication info.

        :returns: Boolean. True if the pathway was updated successfully.
        """
        #self.authInfo
        raise NotImplementedError
        #return self.serv.updatePathway(pwId = pathwayId,
        #description = describeChanges, gpml = gpmlCode, revision = revisionNumb, auth = authInfo)

    def createPathway(self, gpmlCode, authInfo):
        """Create a new pathway on the WikiPathways website with a given GPML code.

        .. warning:: Interface not exposed in bioservices.

        .. note:: To create/modify pathways via the web service, you need to
            have an account with web service write permissions. Please
            contact us to request write access for the web service.

        :param str gpml: The GPML code.
        :param object WSAuth auth: The authentication info.
        :returns: WSPathwayInfo The pathway info for the created pathway
            (containing identifier, revision, etc.).

        """
        raise NotImplementedError
        #return self.serv.createPathway(gpml = gpmlCode, auth = authInfo)

    #def saveCurationTag(self, pathwayId, name, revisionNumb, authInfo, text = None):
    def saveCurationTag(self, pathwayId, name, revisionNumb):
        """Apply a curation tag to a pathway. This operation will overwrite any existing tag with the same name.

        .. warning:: Interface not exposed in bioservices.

        :param str pathwayId: the pathway identifier.
        """
        raise NotImplementedError
        # use the login function to store the authInfo argument
        #if text == None:
        #   res = self.serv.saveCurationTag(pwId = pathwayId, tagName = name, revision = revisionNumb, auth = authInfo)
        #else:
        #   res =  self.serv.saveCurationTag(pwId = pathwayId, tagName = name, tagText = text, revision = revisionNumb, auth = authInfo)
        #return res

    def removeCurationTag(self, pathwayId, name):
        """Remove a curation tag from a pathway.

        .. warning:: Interface not exposed in bioservices.

        """
        #self.authInfo
        raise NotImplementedError
        #return self.serv.removeCurationTag(pwId = pathwayId, tagName = name, auth = authInfo)

    #REST getCurationTags?pwId=WP4
    def getCurationTags(self, pathwayId):
        """Get all curation tags for the given pathway.

        :param str pathwayId: the pathway identifier.
        :returns: Array of WSCurationTag. The curation tags.

        ::

            s.getCurationTags("WP4")
        """
        return self.serv.getCurationTags(pwId = pathwayId)

    # REST getCurationTagsByName?tagName=Curation:FeaturedPathway
    def getCurationTagsByName(self, name):
        """Get all curation tags for the given tag name.

        Use this method if you want to find all pathways that are tagged with a specific curation tag.


        :param str tagName: The tag name.
        :return: Array of WSCurationTag. The curation tags (one instance for each pathway that has been tagged).

        ::

            s.getCurationTagsByName("Curation:FeaturedPathway")
        """
        return self.serv.getCurationTagsByName(tagName = name)

    def getColoredPathway(self, pathwayId, filetype="svg", revision=0):
        """Get a colored image version of the pathway. 


        :param str pwId: The pathway identifier.
        :param int revision: The revision number of the pathway (use '0' for most recent version). 
        :param str fileType:  The image type (One of 'svg', 'pdf' or 'png'). Not
            yet implemented. svg is returned.
        :returns: Binary form of the image.


        .. todo:: graphId, color parameters
        """
        #graphIds, revisionNumb = 0, colors = None, filetype = 'pdf', verbose = False):
        #if colors == None:
        #    colors = 'FF0000'
        #res = self.serv.getColoredPathway(pwId = pathwayId, revision = revisionNumb, graphId = graphIds, color = colors, fileType = filetype)
        res = self.serv.getColoredPathway(pwId=pathwayId, revision=revision)
        #if verbose:
        #    return res
        #else:
        return base64.b64decode(res)

    #def getXrefList(self, pathwayId, sysCode):
    #    """http://www.pathvisio.org/wiki/DatabasesMapps#Supporteddatabasesystems"""
    #    return self.serv.getXrefList(pwId = pathwayId, code = sysCode)

    def findPathwaysByText(self, query, species=None):
        """Find pathways using a textual search on the description and text labels of the pathway objects.

        The query syntax offers several options:

        * Combine terms with AND and OR. Combining terms with a space is equal
          to using OR ('p53 OR apoptosis' gives the same result as 'p53 apoptosis').
        * Group terms with parentheses, e.g. '(apoptosis OR mapk) AND p53'
        * You can use wildcards * and ?. * searches for one or more
          characters, ? searches for only one character.
        * Use quotes to escape special characters. E.g. '"apoptosis*"' will
          include the * in the search and not use it as wildcard.

        This function supports REST-style invocation.
        Example: http://www.wikipathways.org/wpi/webservice/webservice.php/findPathwaysByText?query=apoptosis

        :param str query: The search query (e.g. 'apoptosis' or 'p53').
        :param str species:  The species to limit the search to (leave blank to search on all species).
        :return: Array of WSSearchResult An array of search results.

        ::

            s.findPathwaysByText(query="p53 OR mapk",species='Homo sapiens')
        """
        if species == None:
            return self.serv.findPathwaysByText(query=query)
        else:
            return self.serv.findPathwaysByText(query=query,species=species)

    def getOntologyTermsByPathway(self, pathwayId):
        """Get a list of ontology terms for a given pathway.


        :param str pathwayId: the pathway identifier.
        :return: Array of WSOntologyTerm. The ontology terms.

        ::

            s.getOntologyTermsByPathway("WP4")
        """
        return self.serv.getOntologyTermsByPathway(pwId=pathwayId)

    # REST getOntologyTermsByOntology?ontology=Disease
    def getOntologyTermsByOntology(self, ontologyTerm):
        """Get a list of ontology terms from a given ontology.


        :param str ontology: The ontology term (for possible values, see the Ontology Tags section on the pathway page at WikiPathways website.
        :return: List of WSOntologyTerm The ontology terms.

        ::

            >>> s.getOntologyTermsByOntology("Disease")

        """
        return self.serv.getOntologyTermsByOntology(ontology = ontologyTerm)

    #REST getPathwaysByOntologyTerm?term=DOID:344
    def getPathwaysByOntologyTerm(self, ontologyTermId):
        """Get a list of pathways tagged with a given ontology term.


        :param str ontologyTermId: the ontology term identifier.
        :returns: List of WSPathwayInfo. The pathway information.

        ::

            >>> from bioservices import *
            >>> s = Wikipathway()
            >>> s.getPathwaysByOntologyTerm('DOID:344')

        """
        return self.serv.getPathwaysByOntologyTerm(term=ontologyTermId)

    #REST getPathwaysByParentOntologyTerm?term=DOID:344
    def getPathwaysByParentOntologyTerm(self, ontologyTermId):
        """Get a list of pathways tagged with any ontology term that is the child of the given Ontology term.


        :param str ontologyTermId: the ontology term identifier.
        :returns: List of WSPathwayInfo The pathway information.


        """
        return self.serv.getPathwaysByParentOntologyTerm(term = ontologyTermId)

    def showPathwayInBrowser(self, pathwayId):
        """Show a given Pathway into your favorite browser.

        :param str pathwayId: the pathway identifier.

        .. note: This is an additional bioservices functionality (not a
            wikipathway one) showing a wikipathway URL.

        """
        url = self.serv.getPathwayInfo(pwId=pathwayId).url
        webbrowser.open(url)

